package ch.unige.aou.controller.admin;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.AdminPermissions;
import ch.unige.solidify.security.UserPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.License;
import ch.unige.aou.rest.UrlPath;

@AdminPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@SuppressWarnings("squid:S4684")
@RequestMapping(UrlPath.ADMIN_LICENSES)
public class LicenseController extends ResourceController<License> {
  private static final Logger log = LoggerFactory.getLogger(LicenseController.class);

  @UserPermissions
  @Override
  public HttpEntity<Collection<License>> advancedSearch(@ModelAttribute License license,
          @RequestParam("search") String search,
          @RequestParam(value = "match", required = false) String matchType, Pageable pageable) {
    return super.advancedSearch(license, search, matchType, pageable);
  }

  @Override
  public HttpEntity<License> create(@RequestBody License license) {
    return super.create(license);
  }

  @Override
  public HttpEntity<License> get(@PathVariable String id) {
    return super.get(id);
  }

  @Override
  @GetMapping
  @UserPermissions
  public HttpEntity<Collection<License>> list(@ModelAttribute License search, Pageable pageable) {
    return super.list(search, pageable);
  }

  @Override
  public HttpStatus delete(@PathVariable String id) {
    return super.delete(id);
  }

  @Override
  public HttpStatus deleteList(@RequestBody String[] ids) {
    return super.deleteList(ids);
  }

  @Override
  public HttpEntity<License> update(@PathVariable String id, @RequestBody License license) {
    return super.update(id, license);
  }

}
