package ch.unige.aou.controller.admin;

import java.util.List;

import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.AssociationController;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.NoOnePermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.settings.ResearchGroup;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.security.AdminOrSuperValidatorPermissions;

@RestController
@NoOnePermissions
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_RESEARCH_GROUPS + SolidifyConstants.URL_PARENT_ID + ResourceName.PUBLICATIONS)
public class ResearchGroupPublicationController extends AssociationController<ResearchGroup, Publication> {

  @Override
  public HttpEntity<List<Publication>> create(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.create(parentid, ids);
  }

  @Override
  @AdminOrSuperValidatorPermissions
  public HttpEntity<Collection<Publication>> list(@PathVariable String parentid, @ModelAttribute Publication filterItem, Pageable pageable) {
    return super.list(parentid, filterItem, pageable);
  }

  @Override
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return super.delete(parentid, id);
  }

  @Override
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.deleteList(parentid, ids);
  }

  @Override
  protected String getParentFieldName() {
    return "researchGroups";
  }

  @Override
  protected boolean addChildOnParent(ResearchGroup researchGroup, Publication publication) {
    return researchGroup.addPublication(publication);
  }

  @Override
  protected boolean removeChildFromParent(ResearchGroup researchGroup, Publication publication) {
    return researchGroup.removePublication(publication);
  }
}
