package ch.unige.aou.controller.admin;

import java.util.List;
import java.util.stream.Collectors;

import ch.unige.aou.business.StructureService;
import ch.unige.solidify.exception.SolidifyValidationException;
import ch.unige.solidify.validation.ValidationError;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.AssociationController;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.security.AdminPermissions;

import ch.unige.aou.controller.AdminController;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.rest.UrlPath;
import ch.unige.aou.security.CurrentUserPermissions;

@CurrentUserPermissions
@RestController
@ConditionalOnBean(AdminController.class)
@RequestMapping(UrlPath.ADMIN_STRUCTURES + SolidifyConstants.URL_PARENT_ID + ResourceName.PEOPLE)
public class StructurePersonController extends AssociationController<Structure, Person> {

  @Override
  @AdminPermissions
  public HttpEntity<List<Person>> create(@PathVariable String parentid, @RequestBody String[] personIds) {
    return super.create(parentid, personIds);
  }

  @Override
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id) {
    return super.delete(parentid, id);
  }

  @Override
  @AdminPermissions
  public HttpStatus deleteList(@PathVariable String parentid, @RequestBody String[] ids) {
    return super.deleteList(parentid, ids);
  }

  @Override
  public HttpEntity<Person> get(@PathVariable String parentid, @PathVariable String id) {
    return super.get(parentid, id);
  }

  @Override
  @AdminPermissions
  public HttpEntity<Collection<Person>> list(@PathVariable String parentid, @ModelAttribute Person person, Pageable pageable) {
    return super.list(parentid, person, pageable);
  }

  @Override
  protected String getParentFieldName() {
    return "structures";
  }

  @Override
  protected boolean addChildOnParent(Structure structure, Person person) {
    ((StructureService) this.resourceService).duplicatesInStructures(person, structure);
    return structure.addPerson(person);
  }

  @Override
  protected boolean removeChildFromParent(Structure structure, Person person) {
    return structure.removePerson(person);
  }

}
