package ch.unige.aou.service.admin;

import java.nio.file.Path;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.settings.Institution;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.rest.AouActionName;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.ResourceClientService;

@Service
public class InstitutionClientService extends ResourceClientService<Institution>
        implements InstitutionClientServiceInterface {

  private static final String INSTITUTION_PATH = SolidifyConstants.URL_SEP + ResourceName.INSTITUTIONS;

  public InstitutionClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(INSTITUTION_PATH, Institution.class, ModuleName.ADMIN, restClientTool, env);
  }

  @Override
  public void addPerson(String institutionId, String personId) {
    this.addItem(institutionId, ResourceName.PEOPLE, personId, Person.class);
  }

  @Override
  public List<Institution> findAll() {
    return this.findAll(Institution.class);
  }

  @Override
  public List<Person> getPersons(String institutionId) {
    return this.findAllLinkedResources(institutionId, ResourceName.PEOPLE, Person.class);
  }

  @Override
  public void removePerson(String institutionId, String personId) {
    this.removeItem(institutionId, ResourceName.PEOPLE, personId);
  }

  @Override
  public List<Institution> searchByProperties(Map<String, String> properties) {
    return this.searchByProperties(properties, Institution.class);
  }

  @Override
  public Institution uploadLogo(String resId, Resource file) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP
            + AouActionName.UPLOAD_LOGO;
    return this.uploadFile(url, file, Institution.class);
  }

  @Override
  public void downloadLogo(String resId, Path path) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + AouActionName.DOWNLOAD_LOGO;
    this.downloadByUrlWithToken(url, path);
  }

  @Override
  public void deleteLogo(String resId) {
    final String url = this.resourcePath + SolidifyConstants.URL_SEP + resId + SolidifyConstants.URL_SEP + AouActionName.DELETE_LOGO;
    final RestTemplate restTemplate = this.restClientTool.getClient();
    restTemplate.delete(url);
  }

}
