package ch.unige.aou.service.admin;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.core.env.Environment;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.util.AbstractRestClientTool;

import ch.unige.aou.model.security.Role;
import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.Structure;
import ch.unige.aou.rest.ModuleName;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.ResourceClientService;

@Service
public class StructureClientService extends ResourceClientService<Structure> implements StructureClientServiceInterface {

  public StructureClientService(AbstractRestClientTool restClientTool, Environment env) {
    super(SolidifyConstants.URL_SEP + ResourceName.STRUCTURES, Structure.class, ModuleName.ADMIN, restClientTool, env);
  }

  @Override
  public List<Structure> findAll() {
    return super.findAll(Structure.class);
  }

  @Override
  public List<Structure> searchByProperties(Map<String, String> properties) {
    return super.searchByProperties(properties, Structure.class);
  }

  public Structure update(String resId, String json) {
    final String url = this.getBaseUrl() + this.resourcePath + SolidifyConstants.URL_ID;
    final RestTemplate restTemplate = this.restClientTool.getClient();

    HttpHeaders headers = new HttpHeaders();
    headers.setContentType(MediaType.APPLICATION_JSON);
    headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));

    HttpEntity<String> entity = new HttpEntity<>(json, headers);

    final Map<String, String> params = new HashMap<>();
    params.put(SolidifyConstants.URL_ID_FIELD, resId);
    return restTemplate.patchForObject(url, entity, Structure.class, params);
  }

  @Override
  public List<Person> getPeople(String structureId) {
    return this.findAllLinkedResources(structureId, ResourceName.PEOPLE, Person.class);
  }

  @Override
  public Person getPerson(String structureId, String personId) {
    return this.getChildItemFromRelation(structureId, ResourceName.PEOPLE, Person.class, personId);
  }

  @Override
  public void addPerson(String structureId, String personId) {
    this.addItem(structureId, ResourceName.PEOPLE, personId, Person.class);
  }

  @Override
  public void removePerson(String structureId, String personId) {
    this.removeItem(structureId, ResourceName.PEOPLE, personId);
  }

}
