package ch.unige.aou.controlleradvice;

import static ch.unige.solidify.controller.SolidifyControllerAdvice.StackTraceDisplay.WITH_STACK_TRACE;

import javax.servlet.http.HttpServletRequest;

import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;

import ch.unige.solidify.controller.SolidifyControllerAdvice;
import ch.unige.solidify.repository.SolidifyError;
import ch.unige.solidify.service.MessageService;

import ch.unige.aou.exception.AoUMetadataParseException;
import ch.unige.aou.exception.AouError;
import ch.unige.aou.exception.AouRuntimeException;

@ControllerAdvice
public class AoUControllerAdvice extends SolidifyControllerAdvice {
  private final HttpHeaders headers = new HttpHeaders();

  public AoUControllerAdvice(ErrorAttributes errorAttributes, MessageService messageService) {
    super(errorAttributes, messageService);
    this.headers.add("Content-Type", "application/hal+json");
  }

  @ExceptionHandler(AoUMetadataParseException.class)
  public ResponseEntity<SolidifyError> handleException(AoUMetadataParseException exception, HttpServletRequest request) {
    String message = "";
    if(exception.getCause() != null) {
      message = exception.getCause().getMessage();
    }
    return this.buildResponse(exception, request, message, HttpStatus.BAD_REQUEST, WITH_STACK_TRACE);
  }

  @ExceptionHandler(AouRuntimeException.class)
  public ResponseEntity<AouError> handleException(AouRuntimeException exception, HttpServletRequest request) {
    AouError aouError = exception.getAouError();
    aouError.setPath(request.getRequestURI());

    this.logException(exception, request, WITH_STACK_TRACE);

    aouError.setMessage(exception.getMessage());
    aouError.setError(exception.getMessage());
    aouError.setStatus(HttpStatus.BAD_REQUEST);

    return (ResponseEntity.status(HttpStatus.BAD_REQUEST).headers(this.headers)).body(aouError);
  }
}
