package ch.unige.aou.message;

import java.io.Serializable;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.validation.constraints.NotNull;

import ch.unige.aou.AouConstants;

public class EmailMessage implements Serializable {
  private static final long serialVersionUID = 4919737146903678970L;

  //@formatter:off
  public enum EmailTemplate {
    NEW_SUBSCRIPTION(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Inscription à des notifications / Subscription to notifications"),
    REMOVE_SUBSCRIPTION(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Désinscription à des notifications / Unsubscription to notifications"),
    NEW_DEPOSITS_TO_VALIDATE(AouConstants.NOTIFICATION_EMAIL_VALIDATION_PREFIX + " Nouveaux dépôts à valider"),
    NEW_COMMENT_FOR_VALIDATORS_DEPOSIT(AouConstants.NOTIFICATION_EMAIL_VALIDATION_PREFIX + " Nouveaux commentaires pour les validateurs/trices"),
    NEW_COMMENT_FOR_CONTRIBUTORS_DEPOSIT(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Nouveaux commentaires / New comments"),
    NEW_COMMENT_FOR_DEPOSITOR_DEPOSIT(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Nouveaux commentaires / New comments"),
    MY_INDIRECT_PUBLICATION_VALIDATED(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Nouvelles publications / New publications"),
    MY_PUBLICATIONS_VALIDATED(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Dépôts validés / Approved deposits"),
    MY_DEPOSIT_REQUIRES_FEEDBACK(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Commentaire attendu / Feedback requested"),
    MY_PUBLICATIONS_REJECTED(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Dépôts rejetés / Rejected deposits"),
    FORGOTTEN_PUBLICATIONS_TO_VALIDATE(AouConstants.NOTIFICATION_EMAIL_VALIDATION_PREFIX + " Dépôts en attente de validation (oubliés ?)"),
    FORGOTTEN_PUBLICATIONS_IN_PROGRESS(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Dépôts en cours - en attente d'action de votre part / Deposits in progress - awaiting action on your part"),
    SYNCHRONIZED_RESEARCH_GROUPS(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Groupes de recherche synchronisés"),
    NEW_RESEARCH_GROUPS_TO_VALIDATE(AouConstants.NOTIFICATION_EMAIL_ARCHIVE_OUVERTE_PREFIX + " Nouveaux groupes de recherche à valider");

    private final String subject;

    EmailTemplate(String subject) {
      this.subject = subject;
    }

    public String getSubject() {
      return this.subject;
    }
  }
  // @formatter:on

  @Enumerated(EnumType.STRING)
  @NotNull
  private EmailTemplate template;

  private String to;

  private String[] parameters;

  public EmailMessage(String toRecipient, EmailTemplate template, String[] parameters) {
    this.template = template;
    this.to = toRecipient;
    this.parameters = parameters;
  }

  public EmailTemplate getTemplate() {
    return this.template;
  }

  public void setTemplate(EmailTemplate template) {
    this.template = template;
  }

  public String getTo() {
    return this.to;
  }

  public void setTo(String to) {
    this.to = to;
  }

  public String[] getParameters() {
    return this.parameters;
  }

  public void setParameters(String[] parameters) {
    this.parameters = parameters;
  }

  @Override
  public String toString() {
    return super.toString() + " [template=" + this.template + "]" + " [to=" + this.to + "]";
  }
}
