package ch.unige.aou.model.publication;

import javax.persistence.EmbeddedId;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.rest.JoinResource;
import ch.unige.solidify.util.ReflectionTool;

import ch.unige.aou.model.settings.Structure;

@Entity
@Table(name = "publicationStructures")
public class PublicationStructure extends JoinResource<PublicationStructure, PublicationStructureId> {

  public final static String PATH_TO_PUBLICATION = "compositeKey.publication";
  public final static String PATH_TO_STRUCTURE = "compositeKey.structure";

  public enum LinkType {
    METADATA,   // the publication contains the structure in its metadata
    VALIDATION, // the link with the structure is only used to make the publication visible by validators
  }

  @EmbeddedId
  @JsonIgnore
  private PublicationStructureId compositeKey = new PublicationStructureId();

  @Enumerated(EnumType.STRING)
  @NotNull
  private PublicationStructure.LinkType linkType;

  /****************************************************************/

  public LinkType getLinkType() {
    return this.linkType;
  }

  @Transient
  @JsonIgnore
  public void setPublication(Publication publication) {
    this.compositeKey.setPublication(publication);
  }

  @Transient
  @JsonIgnore
  public void setStructure(Structure structure) {
    this.compositeKey.setStructure(structure);
  }

  @JsonIgnore
  public Publication getPublication() {
    return this.compositeKey.getPublication();
  }

  @JsonIgnore
  public Structure getStructure() {
    return this.compositeKey.getStructure();
  }

  public void setLinkType(LinkType linkType) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), linkType);
  }

  /****************************************************************/

  @Override
  @JsonIgnore
  public PublicationStructureId getCompositeKey() {
    return this.compositeKey;
  }

  @Override
  public void setCompositeKey(PublicationStructureId compositeKey) {
    this.compositeKey = compositeKey;
  }

  @Override
  public String toString() {
    return super.toString() + " (LinkType=" + this.getLinkType() + ")";
  }
}
