package ch.unige.aou.model.settings;

import static ch.unige.aou.AouConstants.DB_DATE_LENGTH;

import java.time.OffsetDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.JoinColumn;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;

import com.fasterxml.jackson.annotation.JsonFormat;

import ch.unige.solidify.util.StringTool;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouResourceNormalized;

/**
 * Global Banner
 */
@Entity
public class GlobalBanner extends AouResourceNormalized<GlobalBanner> {

  public enum GlobalBannerType {
    CRITICAL,
    WARNING,
    INFO
  }

  @NotNull
  @Column(unique = true)
  private String name;

  private Boolean withDescription;

  @Enumerated(EnumType.STRING)
  @NotNull
  private GlobalBanner.GlobalBannerType type;

  @ElementCollection
  @Column(name = AouConstants.DB_GLOBAL_BANNER_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_GLOBAL_BANNER_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_GLOBAL_BANNER_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> titleLabels = new ArrayList<>();

  @ElementCollection
  @Column(name = AouConstants.DB_GLOBAL_BANNER_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_GLOBAL_BANNER_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_GLOBAL_BANNER_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<TextLabel> descriptionLabels = new ArrayList<>();

  @Column(nullable = false)
  private Boolean enabled;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = DB_DATE_LENGTH)
  private OffsetDateTime startDate;

  @JsonFormat(shape = JsonFormat.Shape.STRING, pattern = StringTool.DATE_TIME_FORMAT)
  @Column(length = DB_DATE_LENGTH)
  private OffsetDateTime endDate;

  /***********************************************************/

  public String getName() {
    return this.name;
  }

  public Boolean isWithDescription() {
    return this.withDescription;
  }

  public GlobalBanner.GlobalBannerType getType() {
    return this.type;
  }

  public List<Label> getTitleLabels() {
    return this.titleLabels;
  }

  public List<TextLabel> getDescriptionLabels() {
    return this.descriptionLabels;
  }

  public Boolean getEnabled() {
    return this.enabled;
  }

  public OffsetDateTime getStartDate() {
    return this.startDate;
  }

  public OffsetDateTime getEndDate() {
    return this.endDate;
  }

  /***********************************************************/

  public void setName(String name) {
    this.setCurrentProperty(name);
  }

  public void setWithDescription(Boolean withDescription) {
    this.setCurrentProperty(withDescription);
  }

  public void setType(GlobalBanner.GlobalBannerType type) {
    this.setCurrentProperty(type);
  }

  public void setTitleLabels(Collection<Label> titleLabels) {
    this.setCurrentProperty(titleLabels);
  }

  public void setDescriptionLabels(Collection<TextLabel> descriptionLabels) {
    this.setCurrentProperty(descriptionLabels);
  }

  public void setEnabled(Boolean enabled) {
    this.setCurrentProperty(enabled);
  }

  public void setStartDate(OffsetDateTime startDate) {
    this.setCurrentProperty(startDate);
  }

  public void setEndDate(OffsetDateTime endDate) {
    this.setCurrentProperty(endDate);
  }

  @Override
  public void init() {
    if (this.enabled == null) {
      this.enabled = true;
    }
    if (this.withDescription == null) {
      this.withDescription = false;
    }
  }
}
