package ch.unige.aou.model.settings;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.persistence.CollectionTable;
import javax.persistence.Column;
import javax.persistence.ElementCollection;
import javax.persistence.Entity;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.UniqueConstraint;
import javax.validation.constraints.NotNull;

import org.springframework.data.jpa.domain.Specification;

import ch.unige.solidify.util.ReflectionTool;
import ch.unige.solidify.util.SearchCriteria;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.AouSearchableResourceNormalized;
import ch.unige.aou.model.Sortable;
import ch.unige.aou.specification.LicenseSearchSpecification;

/**
 * Structure based on open licenses. http://licenses.opendefinition.org
 */
@Entity
public class License extends AouSearchableResourceNormalized<License> implements Sortable {

  @ManyToOne
  @NotNull
  private LicenseGroup licenseGroup;

  @NotNull
  @Column(unique = true, nullable = false)
  private String openLicenseId;

  @NotNull
  @Column(unique = true)
  private String title;

  private String version;

  /**
   * Property allowing to sort licenses
   */
  @Column(nullable = false)
  private Integer sortValue;

  private URL url;

  @Column(nullable = false)
  private Boolean visible;

  @ElementCollection
  @Column(name = AouConstants.DB_LICENSE_ID)
  @CollectionTable(joinColumns = { @JoinColumn(name = AouConstants.DB_LICENSE_ID) },
                   uniqueConstraints = @UniqueConstraint(columnNames = { AouConstants.DB_LICENSE_ID, AouConstants.DB_LANGUAGE_ID }))
  private List<Label> labels = new ArrayList<>();

  /***********************************************************/

  public LicenseGroup getLicenseGroup() {
    return this.licenseGroup;
  }

  public String getOpenLicenseId() {
    return this.openLicenseId;
  }

  public String getTitle() {
    return this.title;
  }

  public String getVersion() {
    return this.version;
  }

  @Override
  public Integer getSortValue() {
    return this.sortValue;
  }

  public URL getUrl() {
    return this.url;
  }

  public Boolean getVisible() {
    return this.visible;
  }

  public List<Label> getLabels() {
    return this.labels;
  }

  /***********************************************************/

  public void setLicenseGroup(LicenseGroup licenseGroup) {
    this.setCurrentProperty(licenseGroup);
  }

  public void setOpenLicenseId(String openLicenseId) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), openLicenseId);
  }

  public void setTitle(String title) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), title);
  }

  public void setVersion(String version) {
    this.setCurrentProperty(version);
  }

  @Override
  public void setSortValue(Integer sort) {
    this.setCurrentProperty(sort);
  }

  public void setUrl(URL url) {
    this.setProperty(ReflectionTool.getCurrentSetterVariableName(), url);
  }

  public void setVisible(Boolean visible) {
    this.setCurrentProperty(visible);
  }

  public void setLabels(Collection<Label> labels) {
    this.setCurrentProperty(labels);
  }

  /***********************************************************/

  @Override
  public void init() {
    if (this.visible == null) {
      this.visible = true;
    }
  }

  @Override
  public Specification<License> getSearchSpecification(SearchCriteria criteria) {
    return new LicenseSearchSpecification(criteria);
  }
}
