package ch.unige.aou.model.xml.deposit.v2_4.deserializer;

import java.io.IOException;

import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.ObjectCodec;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;

import ch.unige.solidify.exception.SolidifyRuntimeException;

import ch.unige.aou.model.xml.deposit.AouStdDeserializer;
import ch.unige.aou.model.xml.deposit.v2_4.Collection;
import ch.unige.aou.model.xml.deposit.v2_4.Collections;

public class CollectionsDeserializer extends AouStdDeserializer<Collections> {

  private static final long serialVersionUID = 659865L;

  @Override
  public Collections deserialize(JsonParser parser, DeserializationContext context) throws IOException, JsonProcessingException {

    ObjectCodec codec = parser.getCodec();
    JsonNode jsonNode = codec.readTree(parser);

    if (jsonNode instanceof ArrayNode) {
      if (jsonNode.isEmpty()) {
        return null;
      }

      Collections collections = new Collections();
      for (JsonNode collectionNode : jsonNode) {
        Collection collection = new Collection();

        JsonNode nameNode = collectionNode.get("name");
        JsonNode numberNode = collectionNode.get("number");

        //if list contain one element and all properties of this only element are empty we ignore the list
        if (jsonNode.size() == 1 && !this.nodeHasValue(nameNode) && !this.nodeHasValue(numberNode)) {
          return null;
        }

        if (this.nodeHasTextValue(nameNode)) {
          collection.setName(nameNode.textValue());
        }

        if (this.nodeHasTextValue(numberNode)) {
          collection.setNumber(numberNode.textValue());
        }

        collections.getCollection().add(collection);
      }
      return collections;
    } else {
      throw new SolidifyRuntimeException("collections is not an array");
    }
  }
}
