package ch.unige.aou.specification;

import java.util.List;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import ch.unige.solidify.specification.SolidifySpecification;

import ch.unige.aou.model.publication.DocumentFile;

public class DocumentFileSpecification extends SolidifySpecification<DocumentFile> {
  private static final long serialVersionUID = 4403188963167877868L;

  public DocumentFileSpecification(DocumentFile documentFile) {
    super(documentFile);
  }

  @Override
  protected void completePredicatesList(Root<DocumentFile> root, CriteriaQuery<?> query, CriteriaBuilder builder,
          List<Predicate> predicatesList) {

    if (this.criteria.getDocumentFileType() != null) {
      predicatesList.add(builder.like(root.get("documentFileType").get("resId"), this.criteria.getDocumentFileType().getResId()));
    }
    if (this.criteria.getPublication() != null) {
      predicatesList.add(builder.like(root.get("publication").get("resId"), this.criteria.getPublication().getResId()));
    }
    if (this.criteria.getLicense() != null) {
      predicatesList.add(builder.like(root.get("license").get("resId"), this.criteria.getLicense().getResId()));
    }

    if (this.criteria.getSourceData() != null) {
      predicatesList.add(builder.like(root.get("sourceData"), "%" + this.criteria.getSourceData() + "%"));
    }
    if (this.criteria.getFinalData() != null) {
      predicatesList.add(builder.like(root.get("finalData"), "%" + this.criteria.getFinalData() + "%"));
    }
    if (this.criteria.getFileName() != null) {
      predicatesList.add(builder.like(root.get("fileName"), "%" + this.criteria.getFileName() + "%"));
    }
    if (this.criteria.getMimetype() != null) {
      predicatesList.add(builder.like(root.get("mimetype"), "%" + this.criteria.getMimetype() + "%"));
    }

    if (this.criteria.getStatus() != null) {
      predicatesList.add(builder.equal(root.get("status"), this.criteria.getStatus()));
    }
    if (this.criteria.getFileSize() != null) {
      predicatesList.add(builder.equal(root.get("fileSize"), this.criteria.getFileSize()));
    }
    if (this.criteria.getAccessLevel() != null) {
      predicatesList.add(builder.equal(root.get("accessLevel"), this.criteria.getAccessLevel()));
    }
    if (this.criteria.getEmbargoAccessLevel() != null) {
      predicatesList.add(builder.equal(root.get("embargoAccessLevel"), this.criteria.getEmbargoAccessLevel()));
    }
    if (this.criteria.getEmbargoEndDate() != null) {
      predicatesList.add(builder.equal(root.get("embargoEndDate"), this.criteria.getEmbargoEndDate()));
    }
  }
}
