package ch.unige.aou.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import ch.unige.aou.model.StatusHistory;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.HistoryService;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.CompositionController;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.Tool;
import ch.unige.solidify.security.UserPermissions;

@UserPermissions
public abstract class AouCompositionController<T extends Resource<T>, V extends Resource<V>> extends CompositionController<T, V> {

  @Autowired
  HistoryService historyService;

  @GetMapping(SolidifyConstants.URL_ID_PLUS_SEP + ResourceName.HISTORY)
  public HttpEntity<Collection<StatusHistory>> history(@PathVariable String parentid, @PathVariable String id, Pageable pageable) {
    final V item = this.checkSubresourceExistsAndIsLinkedToParent(id, parentid);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    final Page<StatusHistory> listItem = this.historyService.findByResId(id, pageable);
    final Collection<StatusHistory> collection = new Collection<>(listItem, pageable);
    collection.add(linkTo(this.getClass(), parentid, id, pageable).withSelfRel());
    collection.add(Tool.parentLink((linkTo(this.getClass(), parentid, id, pageable)).toUriComponentsBuilder()).withRel(ActionName.PARENT));
    this.addPageLinks(linkTo(this.getClass(), parentid, id, pageable), collection, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

}
