package ch.unige.aou.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import ch.unige.aou.model.StatusHistory;
import ch.unige.aou.rest.ResourceName;
import ch.unige.aou.service.HistoryService;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.controller.ResourceController;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.Tool;
import ch.unige.solidify.security.UserPermissions;

@UserPermissions
public class AouResourceController<T extends Resource<T>> extends ResourceController<T> {

  @Autowired
  HistoryService historyService;

  @GetMapping(SolidifyConstants.URL_ID_PLUS_SEP + ResourceName.HISTORY)
  public HttpEntity<Collection<StatusHistory>> history(@PathVariable String id, Pageable pageable) {
    final T item = this.itemService.findOne(id);
    if (item == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    final Page<StatusHistory> listItem = this.historyService.findByResId(id, pageable);
    final Collection<StatusHistory> collection = new Collection<>(listItem, pageable);
    collection.add(linkTo(methodOn(this.getClass()).history(id, pageable)).withSelfRel());
    collection
            .add(Tool.parentLink((linkTo(methodOn(this.getClass()).history(id, pageable))).toUriComponentsBuilder()).withRel(ActionName.PARENT));
    this.addPageLinks(linkTo(methodOn(this.getClass()).history(id, pageable)), collection, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

}
