package ch.unige.aou.repository;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import ch.unige.aou.AouConstants;
import ch.unige.aou.model.oai.OAISet;

import ch.unige.solidify.repository.SolidifyRepository;
import ch.unige.solidify.util.StringTool;

@Repository
@SuppressWarnings("squid:S1214")
public interface OAISetRepository extends SolidifyRepository<OAISet> {

  static final Logger log = LoggerFactory.getLogger(OAISetRepository.class);

  // *************
  // ** Queries **
  // *************

  OAISet findByName(String name);

  OAISet findBySpec(String spec);

  // ***************
  // ** Relations **
  // ***************

  // ********************
  // ** Initialization **
  // ********************

  // ********************
  // ** Initialization **
  // ********************

  public default void init() throws Exception {
    this.createOAISet(AouConstants.OAI_ORG_UNIT, "Organizational Units", "Set for all organizational units", "*");
    this.createOAISet(AouConstants.OAI_DOI_REGISTRATION, "DOI Registration", "Set to registry DOIs", "*");
  }

  public default void createOAISet(String spec, String name, String description, String query) throws Exception {
    if (this.findByName(name) == null) {
      OAISet oaiSet = new OAISet();
      oaiSet.setSpec(spec);
      oaiSet.setName(name);
      oaiSet.setQuery(query);
      oaiSet.setEnabled(Boolean.TRUE);
      if (!StringTool.isNullOrEmpty(description)) {
        oaiSet.setDescription(description);
      }
      this.save(oaiSet);
      log.info("OAISet '{}' created", name);
    }
  }
}
