package ch.unige.aou.repository;

import java.util.List;

import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

import ch.unige.solidify.repository.SolidifyRepository;

import ch.unige.aou.model.settings.Person;
import ch.unige.aou.model.settings.ResearchGroup;

public interface ResearchGroupRepository extends SolidifyRepository<ResearchGroup> {

  List<ResearchGroup> findByName(String name);

  // *************
  // ** Queries **
  // *************

  @Query("SELECT u FROM ResearchGroup r JOIN r.people u WHERE r.resId = :parentId AND u.resId=:id")
  Person findPersonByParentId(@Param("parentId") String parentId, @Param("id") String id);

  @Query("SELECT u FROM ResearchGroup r JOIN r.people u WHERE r.resId = :parentId")
  Page<Person> findPersonByParentId(@Param("parentId") String parentId, Pageable pageable);

  @Query("SELECT s FROM User u JOIN u.person p JOIN p.researchGroups s WHERE u.externalUid like :externalUid")
  Page<ResearchGroup> findByUserExternalUid(@Param("externalUid") String externalUid, Pageable pageable);

  // ***************
  // ** Relations **
  // ***************

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> Page<V> findByParentId(String parentId, Class<?> parentClass, Class<V> childClass, Pageable pageable) {
    // ResearchGroup<->User
    if (parentClass.equals(ResearchGroup.class) && childClass.equals(Person.class)) {
      return (Page<V>) this.findPersonByParentId(parentId, pageable);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName() + ")");
  }

  @Override
  @SuppressWarnings({ "squid:S3776", "unchecked" })
  public default <V> V findByParentIdAndChildId(String parentId, Class<?> parentClass, Class<V> childClass, String id) {

    // ResearchGroup<->User
    if (parentClass.equals(ResearchGroup.class) && childClass.equals(Person.class)) {
      return (V) this.findPersonByParentId(parentId, id);
    }
    throw new UnsupportedOperationException(
            "Missing implementation 'findByParentIdAndChildId' method (" + parentClass.getSimpleName() + "->" + childClass.getSimpleName()
                    + ")");
  }

}
