package ch.unige.aou.service;

import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;

import ch.unige.solidify.service.MessageService;

import ch.unige.aou.config.AouProperties;
import ch.unige.aou.message.DocumentFileMessage;
import ch.unige.aou.model.publication.DocumentFile;

@Service
public class DocumentFileJmsEmitter extends JmsService {

  private String documentFilesQueue;

  public DocumentFileJmsEmitter(AouProperties aouProperties, MessageService messageService) {
    super(aouProperties, messageService);

    this.documentFilesQueue = aouProperties.getQueue().getDocumentFiles();
  }

  /**
   * Reacts to DocumentFile events that are sent when DocumentFile are persisted (see DocumentFileListener class)
   *
   * @param documentFile
   */
  @EventListener
  private void manageDocumentFileEvent(DocumentFile documentFile) {
    DocumentFileMessage documentFileMessage = new DocumentFileMessage(documentFile);

    /**
     * If DocumentFile status is RECEIVED, the document file processing must start
     * --> this is done async by sending a JMS to the processing queue
     */
    if (documentFile.getStatus() == DocumentFile.DocumentFileStatus.RECEIVED) {
      this.sendDocumentFileMessageToQueue(documentFileMessage);
    }
  }

  public void sendDocumentFileMessageToQueue(DocumentFileMessage documentFileMessage) {
    this.convertAndSendToQueue(this.documentFilesQueue, documentFileMessage);
  }
}
