package ch.unige.aou.service;

import java.nio.ByteBuffer;
import java.util.UUID;

import org.apache.commons.codec.binary.Base32;
import org.springframework.stereotype.Service;

import ch.unige.aou.AouConstants;
import ch.unige.aou.config.AouProperties;

@Service
public class IdentifierService {

  private String repositoryDoiPrefix;
  private String repositoryPrefix;
  private String repositoryDomain;

  public IdentifierService(AouProperties aouProperties) {
    this.repositoryDoiPrefix = aouProperties.getRepository().getDoiPrefix();
    this.repositoryPrefix = aouProperties.getRepository().getPrefix();
    this.repositoryDomain = aouProperties.getRepository().getDomain();
  }

  /**
   * Compute DOI with
   *
   * @param resId
   * @return DOI
   */
  public String getNewDOI(String resId) {
    return this.repositoryDoiPrefix + this.repositoryPrefix + this.getShortId(resId);
  }

  /**
   * Compute OAI Id
   *
   * @param resId
   * @return OAI Id
   */
  public String getOaiId(String resId) {
    return AouConstants.OAI + AouConstants.OAI_SEP + this.repositoryDomain + AouConstants.OAI_SEP + this.repositoryPrefix + resId;
  }

  /**
   * Extract resId from DOI with
   *
   * @param doi
   * @return resId
   */
  public String getResIdFromDOI(String doi) {
    return this.decodeBase32(doi.substring(doi.lastIndexOf(':')));
  }

  /**
   * Extract resId from OAI ID with
   *
   * @param oaiId
   * @return resId
   */

  public String getResIdFromOaiId(String oaiId) {
    return oaiId.substring(oaiId.lastIndexOf(AouConstants.OAI_SEP) + 1);
  }

  /**
   * Decode resIdInBase32
   *
   * @param resIdInBase32
   * @return resId
   */
  private String decodeBase32(String resIdInBase32) {
    final Base32 decoder = new Base32();
    final byte[] bytes = decoder.decode(resIdInBase32 + "======");
    final ByteBuffer bb = ByteBuffer.wrap(bytes);
    final UUID uuid = new UUID(bb.getLong(), bb.getLong());
    return uuid.toString();
  }

  /**
   * Encode resId in Base32 string
   *
   * @param resId
   * @return resIdInBase32
   */
  private String encodeBase32(String resID) {
    final UUID uuid = UUID.fromString(resID);
    final ByteBuffer bb = ByteBuffer.wrap(new byte[16]);
    bb.putLong(uuid.getMostSignificantBits());
    bb.putLong(uuid.getLeastSignificantBits());
    final Base32 encoder = new Base32();
    final String base32Value = encoder.encodeAsString(bb.array());
    // Remove 6 '=' at the end
    return base32Value.substring(0, base32Value.length() - 6).toLowerCase();
  }

  /**
   * Compute short Id
   *
   * @param resId
   * @return shortId
   */
  private String getShortId(String resId) {
    return this.encodeBase32(resId);
  }
}
