package ch.unige.aou.service.metadata;

import java.util.List;
import java.util.stream.IntStream;

import org.springframework.stereotype.Service;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;

import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.util.StringTool;

import ch.unige.aou.business.DocumentFileService;
import ch.unige.aou.business.PublicationSubSubtypeService;
import ch.unige.aou.business.PublicationSubtypeService;
import ch.unige.aou.business.PublicationTypeService;
import ch.unige.aou.business.ResearchGroupService;
import ch.unige.aou.business.StructureService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.Publication;
import ch.unige.aou.model.tool.ValidationTool;
import ch.unige.aou.service.rest.DuplicateService;

@Service
public class MetadataValidatorV1 extends MetadataValidatorService {

  public MetadataValidatorV1(AouProperties aouProperties, MetadataExtractorV1 metadataExtractor, MessageService messageService,
          PublicationTypeService publicationTypeService, PublicationSubtypeService publicationSubtypeService,
          PublicationSubSubtypeService publicationSubSubtypeService, ResearchGroupService researchGroupService,
          StructureService structureService, DuplicateService duplicateService, DocumentFileService documentFileService) {
    super(aouProperties, metadataExtractor, messageService, publicationTypeService, publicationSubtypeService, publicationSubSubtypeService,
            researchGroupService, structureService, duplicateService, documentFileService);
  }

  @Override
  protected void validateFormDescriptionStep(Publication publication, Object depositDocObj, BindingResult errors) {
    super.validateFormDescriptionStep(publication, depositDocObj, errors);

    this.validateLanguage(publication, depositDocObj, errors);
    this.validateContainerISSN(publication, depositDocObj, errors);
    this.validateContainerISBN(publication, depositDocObj, errors);
    this.validateDOIs(publication, depositDocObj, errors);
  }

  @Override
  protected void validateLanguage(Publication publication, Object depositDocObj, BindingResult errors) {
    String language = ((MetadataExtractorV1) this.metadataExtractor).getLanguage(depositDocObj);
    if (StringTool.isNullOrEmpty(language)) {
      errors.addError(new FieldError(publication.getClass().getSimpleName(), FORM_STEP_DESCRIPTION_PREFIX + "language",
              this.messageService.get(ERROR_FIELD_CANNOT_BE_EMPTY)));
    }
  }

  protected void validateContainerISSN(Publication publication, Object depositDocObj, BindingResult errors) {
    String issn = ((MetadataExtractorV1) this.metadataExtractor).getContainerISSN(depositDocObj);
    if (!StringTool.isNullOrEmpty(issn) && !ValidationTool.isValidISSN(issn)) {
      errors.addError(new FieldError(publication.getClass().getSimpleName(), FORM_STEP_DESCRIPTION_PREFIX + "identifiers.issn",
              this.messageService.get(ERROR_INVALID_ISSN)));
    }
  }

  protected void validateContainerISBN(Publication publication, Object depositDocObj, BindingResult errors) {
    String isbn = ((MetadataExtractorV1) this.metadataExtractor).getContainerISBN(depositDocObj);
    if (!StringTool.isNullOrEmpty(isbn) && !ValidationTool.isValidISBN(isbn)) {
      errors.addError(new FieldError(publication.getClass().getSimpleName(), FORM_STEP_DESCRIPTION_PREFIX + "identifiers.isbn",
              this.messageService.get(ERROR_INVALID_ISBN)));
    }
  }

  protected void validateDOIs(Publication publication, Object depositDocObj, BindingResult errors) {
    List<String> dois = ((MetadataExtractorV1) this.metadataExtractor).getDOIs(depositDocObj);
    if (dois != null) {
      IntStream.range(0, dois.size()).forEach(index -> {
        String doi = dois.get(index);
        // TODO : check that doi doesn't already exist in existing deposits. Use ElasticSearch for that ?
        if (!StringTool.isNullOrEmpty(doi) && !ValidationTool.isValidDOI(doi)) {
          errors.addError(
                  new FieldError(publication.getClass().getSimpleName(), FORM_STEP_DESCRIPTION_PREFIX + "identifiers.dois.doi." + index + ".",
                          this.messageService.get(ERROR_INVALID_DOI)));
        }
      });
    }
  }
}
