package ch.unige.aou.service.metadata.imports;

import java.net.URI;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.unige.solidify.service.MessageService;

import ch.unige.aou.business.ContributorService;
import ch.unige.aou.business.DocumentFileService;
import ch.unige.aou.business.DocumentFileTypeService;
import ch.unige.aou.business.LicenseService;
import ch.unige.aou.config.AouProperties;
import ch.unige.aou.model.publication.DocumentFile;
import ch.unige.aou.model.publication.DocumentFileType;
import ch.unige.aou.model.xml.deposit.v2_4.AccessLevel;
import ch.unige.aou.model.xml.deposit.v2_4.DepositDoc;
import ch.unige.aou.model.xml.deposit.v2_4.FileType;
import ch.unige.aou.service.MetadataService;
import ch.unige.aou.service.rest.JournalTitleRemoteService;

public abstract class MetadataAndFileImportService extends MetadataImportService {

  private static final Logger log = LoggerFactory.getLogger(MetadataAndFileImportService.class);

  private DocumentFileService documentFileService;
  protected DocumentFileTypeService documentFileTypeService;

  protected List<String> ignoredFilesSourceUrlPatterns;

  protected MetadataAndFileImportService(AouProperties aouProperties, MessageService messageService, MetadataService metadataService,
          ContributorService contributorService, DocumentFileService documentFileService, DocumentFileTypeService documentFileTypeService,
          JournalTitleRemoteService journalTitleRemoteService, LicenseService licenseService) {
    super(aouProperties, messageService, metadataService, contributorService, journalTitleRemoteService, licenseService);

    this.documentFileService = documentFileService;
    this.documentFileTypeService = documentFileTypeService;

    this.ignoredFilesSourceUrlPatterns = List.of(aouProperties.getMetadata().getImports().getIgnoredFilesSourceUrlPatterns());
  }

  protected DocumentFileType convertFileTypeDocToDocumentFileType(FileType type) {
    return this.documentFileTypeService.findByValue(type.value());
  }

  protected DocumentFile.AccessLevel convertAccessLevel(AccessLevel accessLevel) {
    switch (accessLevel) {
      case PUBLIC:
        return DocumentFile.AccessLevel.PUBLIC;
      case RESTRICTED:
        return DocumentFile.AccessLevel.RESTRICTED;
      case PRIVATE:
        return DocumentFile.AccessLevel.PRIVATE;
      default:
        return null;
    }
  }

  protected void ensureFileExists(DepositDoc depositDoc) {
    if (depositDoc.getFiles() == null) {
      depositDoc.setFiles(this.factory.createFiles());
    }
  }

  /**
   * Check if the URL must be ignored when fetching document files from external services
   *
   * @param fileSourceUri URL of the file to download
   * @return
   */
  private boolean ignoreDocumentFileSourceDataUrl(URI fileSourceUri) {
    return fileSourceUri == null
            || this.ignoredFilesSourceUrlPatterns.stream().anyMatch(ignoredUrl -> fileSourceUri.toString().matches(ignoredUrl));
  }

  protected void saveDocumentFileIfSourceDataValid(DocumentFile documentFile) {
    if (!this.ignoreDocumentFileSourceDataUrl(documentFile.getSourceData())) {
      this.documentFileService.save(documentFile);
    } else {
      log.info("DocumentFile sourceData '{}' belongs to ignored file source urls. DocumentFile won't be created", documentFile.getSourceData());
    }
  }
}
