package ch.unige.aou.test.resource;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.unige.aou.rest.UrlPath;

public class OaiSetDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getName() {
    return UrlPath.ACCESS_OAI_SETS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    final List<FieldDescriptor> documentationFields = new ArrayList<>(Arrays.asList(
            // @formatter:off
      new FieldDescriptorBuilder(this)
        .name("spec")
        .description("The specification of the OAI set")
        .mandatory(true)
        .type(TypeField.STRING)
        .customConstraints("In lower case")
        .build(),
      new FieldDescriptorBuilder(this)
        .name("name")
        .description("The name of the OAI set")
        .mandatory(true)
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("description")
        .description("The description of the OAI set")
        .type(TypeField.STRING)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("query")
        .description("The query of the OAI set")
        .mandatory(true)
        .type(TypeField.STRING)
        .customConstraints("Query compliante with indexing search")
        .testValue(TEST_SEARCH_QUERY)
        .build().type(JsonFieldType.STRING),
      new FieldDescriptorBuilder(this)
        .name("enabled")
        .description("If the OAI set is enable")
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN)));
    // @formatter:on
    return documentationFields;
  }
}
