package ch.unige.aou.test.resource;

import static org.springframework.restdocs.payload.PayloadDocumentation.subsectionWithPath;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.payload.JsonFieldType;
import org.springframework.restdocs.snippet.Attributes;

import ch.unige.solidify.test.helper.AbstractResourceDoc;
import ch.unige.solidify.test.helper.FieldDescriptorBuilder;
import ch.unige.solidify.test.model.TypeField;

import ch.unige.aou.model.security.User;
import ch.unige.aou.rest.UrlPath;

public class UserDoc extends AbstractResourceDoc {

  @Override
  public List<String> getListLinks() {
    return Collections.emptyList();
  }

  @Override
  public String getModelName() {
    return getCamelCase(User.class.getSimpleName());
  }

  @Override
  public String getName() {
    return UrlPath.ADMIN_USERS;
  }

  @Override
  public List<String> getResourceLinks() {
    return Collections.emptyList();
  }

  @Override
  protected List<FieldDescriptor> getDocumentationFields() {
    // @formatter:off
    return Arrays.asList(
      new FieldDescriptorBuilder(this)
        .name("firstName")
        .description("The first name of the user")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastName")
        .description("The last name of the user")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("fullName")
        .description("The full name of the user")
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("externalUid")
        .description("The external UID of the user")
        .mandatory(true)
        .type(TypeField.STRING)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("email")
        .description("The email of the user")
        .mandatory(true)
        .type(TypeField.STRING)
        .customConstraints("Valid email address")
        .testValue(TEST_EMAIL_VALUE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("homeOrganization")
        .description("The home organization of the user")
        .mandatory(true)
        .type(TypeField.STRING)
        .testValue(TEST_STRING_VALUE)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("enabled")
        .description("If the user is enable")
        .mandatory(true)
        .type(TypeField.BOOLEAN)
        .build().type(JsonFieldType.BOOLEAN),
      new FieldDescriptorBuilder(this)
        .name("person")
        .description("The linked person of the user")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastLoginTime")
        .description("Time of last login")
        .type(TypeField.UNDEFINED)
        .build(),
      new FieldDescriptorBuilder(this)
        .name("lastLoginIpAddress")
        .description("The IP address from last login")
        .build(),
      subsectionWithPath("applicationRole")
        .description("The application role details of the user")
        .attributes(new Attributes.Attribute(READ_ONLY_VALUE, String.valueOf(true))));
  // @formatter:on

  }

}
